

# Global variables to store translations and fallback language
_translations = {}
_config = {
    "silent_kwargs": False,
    "language": "",
    "fallback_language": "",
}


def set_config(config_key: str, value: str) -> None:
    '''
    Gives write access to the configuration of translatium.

    Parameters:
    - config_key: The key of the configuration to change
    - value: The value to set

    Returns: None
    '''
    global _config
    _config[config_key] = value
    return None

def get_config() -> dict:
    '''
    Gives read access to the configuration of translatium.

    Returns: A dictionary with the configuration
    '''
    global _config
    return _config

def set_translations(translations: dict) -> None:
    '''
    Sets the translations for translatium.

    Parameters:
    - translations: A dictionary with the translations

    Returns: None
    '''
    global _translations
    _translations = translations
    return None

def get_translations() -> dict:
    '''
    Gets the translations for translatium.

    Returns: A dictionary with the translations
    '''
    global _translations
    return _translations





class TranslationError(Exception):
    def __init__(self, message, value=""):
        self.message = message
        self.value = value
        super().__init__(self.message)
    def __str__(self):
        return f'{self.message} -> {self.value}'



import sys
import pytest
from pathlib import Path

sys.path.insert(0, str(Path(__file__).resolve().parent.parent / 'src'))

import translatium

# Get absolute path to locales directory
LOCALES_PATH = Path(__file__).resolve().parent / 'locales'



def change_translations_module_scope(monkeypatch, data: dict):
    monkeypatch.setattr(translatium.config, '_translations', data)



############################################################
#                      # ACTUAL TESTS                      #
############################################################

@pytest.mark.dependency()
def test_import():
    """Test that the module can be imported"""
    assert translatium is not None

@pytest.mark.dependency(depends=["test_import"])
def test_init():
    """Test initializing with locales path and fallback language"""
    translatium.init_translatium(LOCALES_PATH, 'en_US')
    
@pytest.mark.dependency(depends=["test_init"])
def test_set_language():
    """Test setting the active language"""
    translatium.set_config('language', 'de_DE')
    assert translatium.get_config()["language"] == 'de_DE'

@pytest.mark.dependency(depends=["test_set_language"])
def test_translations():
    """Test that translations work correctly"""
    # Initialize first
    translatium.init_translatium(LOCALES_PATH, 'en_US')
    translatium.set_config('language', 'de_DE')
    
    # Test German translation
    assert translatium.translation('hello_message', name="Louis") == 'Hallo Welt! Louis'
    
    # Test fallback to English
    translatium.set_config("language", 'invalid')
    assert translatium.translation('hello_message', name="Louis") == 'Hello World! Louis'

@pytest.mark.dependency(depends=["test_translations"])
def test_more_depth_in_translations(monkeypatch):
    translatium.init_translatium(LOCALES_PATH, 'en_US')
    translatium.set_config('language', 'de_DE')
    data= {
        "en_US": {
            "mail": {
                "one": "You have one new mail",
                "many": "You have many new mails"
            }
        },
        "de_DE": {
            "mail": {
                "one": "Sie haben eine neue E-Mail",
                "many": "Sie haben viele neue E-Mails"
            }
        }
    }
    change_translations_module_scope(monkeypatch, data)
    assert translatium.translation('mail.one') == 'Sie haben eine neue E-Mail'
    assert translatium.translation('mail.many') == 'Sie haben viele neue E-Mails'
    translatium.set_config("language", 'invalid')
    assert translatium.translation('mail.one') == 'You have one new mail'
    assert translatium.translation('mail.many') == 'You have many new mails'



import yaml
from pathlib import Path
from .errors import TranslationError
from .config import set_translations, get_translations, set_config, get_config
from .utils import deprecated

def init_translatium(path: Path, fallback: str) -> None:
    '''
    Initialize translatium. It does automatically load the translations and checks for errors.

    Parameters:
    - path: Path to the directory containing the translation files
    - fallback: The fallback language to use when a translation is not found in the selected language

    Returns: None
    '''
    set_translations(load_translations(path))
    set_config("fallback_language", fallback)
    checks()
    return None

def checks() -> None:
    '''
    Check if the translations are valid and if the fallback language is available.

    Returns: None
    '''
    # Check if fallback language is available
    def check_fallback_language():
        if get_config()["fallback_language"] not in get_translations():
            raise TranslationError("Fallback Language not found", get_config()["fallback_language"])
    # Check if all keys in the languages (except the fallback language) are present in the fallback language
    def check_translation_keys():
        fallback_keys = set(get_translations()[get_config()["fallback_language"]].keys())
        for lang, translations in get_translations().items():
            if lang == get_config()["fallback_language"]:
                continue
            missing_keys = set(translations.keys()) - fallback_keys
            if missing_keys:
                raise TranslationError(f"Translation keys {missing_keys} in language '{lang}' not found in fallback language", missing_keys)
    check_fallback_language()
    check_translation_keys()
    return None

@deprecated("Use set_config instead", "v0.3.0")
def set_language(language: str) -> None:
    '''
    Sets the preferred language for translations.
    If the language is not available, an error is raised.
    If the language is set to "invalid", the laguage is changed to the string invalid. This doen for testing purposes.

    Parameters:
    - language: The language code to set as the preferred language

    Returns: None
    '''
    # Check if the language is available
    if language == "invalid":
        set_config("language", language)
    elif language not in get_translations():
        raise TranslationError("Language not found", language)
    else:
        set_config("language", language)
    return None

def translation(translation_key: str, **kwargs) -> str:
    '''
    Gets the translation for a specific key in the selected language.
    If the translation is not found in the selected language, the fallback language is used.

    Parameters:
    - translation_key: The key for the translation

    Returns: A string with the translation
    '''
    # Helper function to get translation from a specific language
    def get_translation(language, keys):
        translation = get_translations().get(language, {})
        for key in keys:
            if isinstance(translation, dict):
                translation = translation.get(key)
            else:
                return None
        return translation
    keys = translation_key.split('.')
    translation = get_translation(get_config()["language"], keys) or get_translation(get_config()["fallback_language"], keys)
    if translation:
        return translation.format(**kwargs)
    else:
        raise TranslationError(
            f"Translation key '{translation_key}' not found in selected language '{get_config()["language"]}' or fallback language '{get_config()["fallback_language"]}'", translation_key)

def load_translations(path: Path) -> dict:
    '''
    Loads translations from the specified directory.

    Parameters:
    - path: Path to the directory containing the translation files

    Returns: A dictionary with the translations
    '''
    translations = {}
    for file in path.glob('*.yaml'):
        lang_code = file.stem  # Extract the language code from the filename
        with file.open('r') as f:
            translations[lang_code] = yaml.safe_load(f)
    for file in path.glob('*.yml'):
        lang_code = file.stem  # Extract the language code from the filename
        with file.open('r') as f:
            translations[lang_code] = yaml.safe_load(f)
    return translations



from pathlib import Path

import setuptools

VERSION = "0.0.2"  # PEP-440
NAME = "translatium"
DESCRIPTION = "A pure Python i18n library for your Python projects."
URL="https://github.com/CEOXeon/Translatium"
PROJECT_URLS = {
    "Source Code": "https://github.com/CEOXeon/Translatium",
}
AUTHOR = "Louis Zimmermann (@CEOXeon)"
AUTHOR_EMAIL = "louis-github@tutanota.com"
LICENSE = "Apache-2.0"
CLASSIFIERS = [
    "Development Status :: 2 - Pre-Alpha",
    "Intended Audience :: Developers",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.12",
]
INSTALL_REQUIRES = [
    "pyyaml",
]

setuptools.setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    url=URL,
    project_urls=PROJECT_URLS,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license=LICENSE,
    classifiers=CLASSIFIERS,
    #python_requires=">=3.8",
    # Requirements
    install_requires=INSTALL_REQUIRES,
    packages=setuptools.find_packages(where="src"),
    package_dir={"": "src"},
    long_description=Path("README.md").read_text(),
    long_description_content_type="text/markdown",
)
